/*************************************************************************
 *                  GENERATEUR D'IMPULSIONS Par B.LECLERC
 *                  
 *  Générateur de signaux logiques au niveau TTL, Sortie tension variable 
 *  et  TTL complémenté,  ajustable sur le temps d'état "HAUT" et 
 *  indépendament d'états "BAS" ainsi que du nombre de pulses à générer                    
 *  de 1 à 999 pulses de 1 miliseconde à 999 secondes sur chaque états.
 *  Avce posibilité de mémorisation et d'attribution des même valeurs
 *  plusieurs fois de suite; ou simplement d'un fonctionnement en continu .
 */
 
#include <EEPROM.h>  //Declaration de l'EEPROM et de ses librairies 
#include "LiquidCrystal_I2C.h"
LiquidCrystal_I2C lcd(0x27, 16, 2); // LCD Initialise


byte SW = 4;// Validation par appui sur le bouton du codeur...
byte DT = 2;// Pulse encodeur
byte CLK = 3;// Synchro encodeur

byte N_OUT = 9;// Sortie complémentée
byte OUT = 10; // Sortie sur tension réglable
byte TTL_OUT = 11; // Sortie TTL

int pos = 0; //Variable de position affichage
int n; //Variable d'incrément codeur
int uu, ud, uc; // variables de temps Hi
int lu, ld, lc; // variables de temps Low
int multi; // Variable multiplicateur ms <=> s
int TimHi, TimLo; //  Variable du temps Signal haut,signal bas 
int xFois; //Boucle du nombre d'impulsion
int uFois,dFois,cFois; //variables du nombre d'impulsions

unsigned char menuState = 0; // Pages du menu

int ms = 0; //Secondes "High" ou miliseconde "Low"

bool clkState  = HIGH; //Etat impulsion encodeur
bool clkLast  = LOW;  //Etat dernière impulsion encodeur


void setup() {
  lcd.clear();
  lcd.begin(); //Initialise
  lcd.backlight(); // Allume le retro-éclairage
  lcd.home();
  lcd.print("T-Hi: T-Lo: Nb:");
  lcd.setCursor(0, 1);
  lcd.blink();  // Clignotement du caractère de saisie 




pinMode(SW, INPUT_PULLUP);//Direction et attribution des pins
pinMode(DT, INPUT);
pinMode(CLK, INPUT);

pinMode(N_OUT, OUTPUT);
pinMode(OUT, OUTPUT);
pinMode(TTL_OUT, OUTPUT);

}

void loop() {
   clkState = digitalRead(CLK);  // Rotation ??
  if ((clkLast == HIGH) && (clkState == LOW)) { //Changement d'état ...
    if (digitalRead(DT) == HIGH) { // Sens horaire...
      n++;  //incrémentation du chiffre
            if ( n > 9 ) {  // limitation du chiffre a 9
        n = 0;
      }
    }
    else {
      n--;  //Sens anti-horraire decrementation du chiffre
      if ( n < 0 ) {  // limitation a 0
        n = 9;
      }
    }
  lcd.setCursor(pos, 1);
  lcd.print(n); // Affichage du chiffre traité 
    delay(20);
  }

  clkLast = clkState; // mémorisation de l'ancienne position encodeur

   if (digitalRead(SW) == LOW) { // Si le bouton est pressé,
      delay(200);

  switch (menuState) { // Suivant le numéros du menu
    case 0: { // Premier menu...
      uc = n; // attribution de la valeur encodeur aux centaines
      pos = 1; //Attrbution de la position d'affichage 
      n = 0; // RàZ de la valeur encodeur
      menuState = 1; //Incrémentation du menu
      delay(300);
              } break; // fin du cas...
       
    case 1: {
      ud = n;
      pos = 2;
      n = 0;
      menuState = 2;
      delay(300);
      } break;      
      
    case 2: {
      uu = n;
      pos = 4;
      n = 0;
      menuState = 3;
      delay(300);
      } break;

    case 3: {
    ms = n;
    if (ms <1 ) {  //Choix entre mllisecondes 
      multi = 1;
     lcd.setCursor(4, 1);
      lcd.print("mS");
    }
    else {
      multi = 1000; // et secondes 
    lcd.setCursor(4, 1);
      lcd.print("S");
    }
      pos = 7;
      n = 0;
      menuState = 4;
      delay(300);
          } break; 
    
    case 4: {
      lc = n;
      pos = 8;
      n = 0;
      menuState = 5;
      delay(300);
          } break;

    case 5: {
      ld = n;
      pos = 9;
      n = 0;
      delay(300);
      menuState = 6;
          } break;

    case 6: {
      lu = n;
      pos = 12;
      n = 0;
      delay(300);
      menuState = 7;
          } break;

    case 7: {
      cFois = n;
      pos = 13;
      n = 0;
      delay(300);
      menuState = 8;
          } break;

   case 8: {
      dFois = n;
      pos = 14;
      n = 0;
      delay(300);
      menuState = 9;
          } break;

    case 9: {
      uFois = n;
      lcd.noBlink(); //fin des saisies => arret clignotement case !
      pos = 15;
      n = 0;
      delay(300);
      menuState = 10;
          } break;

    case 10: {
      delay(300);
      calcul();  // Validation pour aller au calcul des valeurs des puls ...
          } break;
  }
}
}

void calcul() {
  // Calcul des temps des états logiques et du nombre des puls à générer  
xFois = (100*cFois)+(dFois * 10) + uFois ; //Addition des centaines, dizaines, et unités pour definir le Nb de puls à générer
TimHi = ((100*uc)+(10*ud)+uu)*multi ;  //Addition des centaines, dizaines, et unités pour definir la durée de l'atat logique "haut"
TimLo = ((100*lc)+(10*ld)+lu)*multi ;  //Addition des centaines, dizaines, et unités pour definir la durée de l'atat logique "bas"
EEPROM.put(10, xFois); // Stockage en Eeprom du Nb de puls à générer dans la case 10
EEPROM.put(15, TimHi);  // Stockage en Eeprom la durée de l'atat logique "haut" dans la case 15
EEPROM.put(20, TimLo); //  Stockage en Eeprom la durée de l'atat logique "bas" dans la case 20
  if (xFois > 0) { // Si la valeur du Nb de puls est défini, on va sur la boucle de décomptage
    deCompt();
  }
  else {  // si aucun nB de puls n'est attibué, on va sur la boucle éternelle ...
 mouLine();
  }
}

void mouLine() { 
  // Gestion des sorties sans nombre de puls programmé, sans fin ...
  digitalWrite(N_OUT,LOW); // Etat logique complémenté 
digitalWrite(OUT,HIGH);  // Etat logique sortie variable
digitalWrite(TTL_OUT,HIGH);// Etat logique sortie TTL
  delay(TimHi); //Temporisation en état logique "haut"
  digitalWrite(N_OUT,HIGH); // Etat logique complémenté 
digitalWrite(OUT,LOW);  // Etat logique sortie variable
digitalWrite(TTL_OUT,LOW);  // Etat logique sortie TTL
  delay(TimLo); //Temporisation en état logique "bas"
mouLine(); //Ad Vitam eternam
}

void deCompt() {
  //Boucle de mise à l'état des Sorties sur Nb de fois demandé
  digitalWrite(N_OUT,LOW); // Etat logique complémenté 
digitalWrite(OUT,HIGH);  // Etat logique sortie variable
digitalWrite(TTL_OUT,HIGH);  // Etat logique sortie TTL
  delay(TimHi); //Temporisation en état logique "haut"
  digitalWrite(N_OUT,HIGH); // Etat logique complémenté 
digitalWrite(OUT,LOW);  // Etat logique sortie variable
digitalWrite(TTL_OUT,LOW);  // Etat logique sortie TTL
  delay(TimLo); //Temporisation en état logique "bas"
    xFois--; // Décrémentation du nombre de fois
  lcd.setCursor(12, 1);
  lcd.print(xFois); //Affichage du nombre de fois
    lcd.print("  "); // Masque lors du passage à < 1 des centaines et dizaines
    if (xFois < 1){ // Si le nombre de fois est à 0 
      aFini(); // On va sur "Fini"
    }
  else { //Sinon on continue le décompte en gérant les Sorties
    deCompt();  
  }
}

void aFini() {
  // Boucle de mise en Stand by (sauf si nouvelle demande )
delay (1000);
  if (xFois < 1) { // Si le nombre de fois est écoulé
    lcd.noBacklight(); // Mise au repos et extinction de l'éclairage
  }
  if (digitalRead(SW) == LOW){ //Si on appui sur le bouton
    rAppel(); //Nouvelle demande avec les même valeurs
    delay(300);
  }
  aFini(); //On boucle sur la fin sauf si une demande revient
}   

void rAppel() {
  // Réatribution des dernières valeurs et relance du générateur de puls
lcd.backlight(); //Allume le rétro_éclairage
EEPROM.get(10, xFois);  // lecture et affectation dela valeur de la case 10 à Nombre de fois
EEPROM.get(15, TimHi); // lecture et affectation dela valeur de la case 15 à Etat haut
EEPROM.get(20, TimLo);  // lecture et affectation dela valeur de la case 20 à Etat bas
    deCompt(); 
}
